/*
    Copyright (C) 2004, 2009 Vansteenhuyse Hubert <hvansteenhuyse@yahoo.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#ifndef __LEPTON_XML_H__
#define __LEPTON_XML_H__
#ifdef __cplusplus
	 extern "C" {
#endif

#define LEPTON_XML_PARSER_VERSION "1.2"

typedef enum{
	XML_TAG = 1,
	XML_TAG_CLOSE,
	XML_ATTRIB
}XML_DATA_TYPE;

typedef struct XML_DATA{
	size_t group_id, line;
	XML_DATA_TYPE type;
	unsigned char *name, *val;
	struct XML_DATA *parent;
	int sw_tag_open;
}XML_DATA;

typedef struct LX_PARSER{
		int sw_error;
        unsigned char *buf, *idx;
		XML_DATA **data;
		size_t datacounter, bufsze, line;
		void *ExtenalData;

		//let the following pointers point to your routines...
		//return 1 if you want the parser to stop, else return 0!
		int (*OnBegin)(struct LX_PARSER*);
		int (*OnFinnish)(struct LX_PARSER*);

		//content is parsed data unless <![CDATA[ ... ]]> was specified, cdata tags are copied as is in content!
		int (*OnTag)(struct LX_PARSER*, unsigned char *tagname, unsigned char *content);
		int (*OnTagEnd)(struct LX_PARSER*, unsigned char *tagname);
		int (*OnAttribute)(struct LX_PARSER*, unsigned char *tagname, unsigned char *attname, unsigned char *attvalue);
		void (*OnError)(struct LX_PARSER*, size_t line, char *error);
}LX_PARSER;

//other encodings can use these methods to assign a new buffer
void lt_set_name(XML_DATA *data, unsigned char *name);
void lt_set_value(XML_DATA *data, unsigned char *value);

LX_PARSER* lt_new();
LX_PARSER* lt_delete(LX_PARSER*);

int lt_loadfile(LX_PARSER*, char *path);    //returns 0 on error
int lt_loadfilef(LX_PARSER *lx, FILE *f);

void lt_parse(LX_PARSER*);

#ifdef __cplusplus
	 } /*extern "C" */
#endif

#endif //__LEPTON_XML_H__
