/*
    Copyright (C) 1997, 2009 Vansteenhuyse Hubert <hvansteenhuyse@yahoo.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef LT_WRAPPER_HPP_
#define LT_WRAPPER_HPP_
#include "lt_parser.h"

#ifdef __cplusplus
	 extern "C" {
#endif
/*
 * forward declarations
 */
int OnBegin(LX_PARSER*);
int OnFinnish(LX_PARSER*);
int OnTag(LX_PARSER*, unsigned char*, unsigned char*);
int OnTagEnd(LX_PARSER*, unsigned char*);
int OnAttribute(LX_PARSER*, unsigned char*, unsigned char*, unsigned char*);
void OnError(LX_PARSER*, size_t, char*);


/*
 * The Virtual C++ wrapper class.
 * Just implement the virtual methods in your child class.
 */
class LTPARSER{
private:
	LX_PARSER *mLX;

protected:
	int loadfile(char *path){
		return ::lt_loadfile(this->mLX, path);
	}

	void parse(){
		::lt_parse(this->mLX);
	}
public:
	LTPARSER(){
		this->mLX = ::lt_new();
		if(this->mLX){
			this->mLX->ExtenalData = this;
			this->mLX->OnBegin = ::OnBegin;
			this->mLX->OnFinnish = ::OnFinnish;
			this->mLX->OnTag = ::OnTag;
			this->mLX->OnTagEnd = ::OnTagEnd;
			this->mLX->OnAttribute = ::OnAttribute;
			this->mLX->OnError = ::OnError;
		}
	}

	virtual ~LTPARSER(){
		this->mLX = ::lt_delete(this->mLX);
	}

	virtual int OnBegin(LTPARSER *lxp) = 0;
	virtual int OnFinnish(LTPARSER *lxp) = 0;
	virtual int OnTag(LTPARSER *lxp, unsigned char *tagname, unsigned char *content) = 0;
	virtual int OnTagEnd(LTPARSER *lxp, unsigned char *tagname) = 0;
	virtual int OnAttribute(LTPARSER *lx, unsigned char *tagname, unsigned char *attname, unsigned char *attvalue) = 0;
	virtual void OnError(LTPARSER *lxp, size_t line, char *error) = 0;
};

/*
 * C callback routines
 */
int OnBegin(LX_PARSER *lx){
	LTPARSER *pClass;
	if(lx && lx->ExtenalData){
		pClass = (LTPARSER*)lx->ExtenalData;
		return pClass->OnBegin(pClass);
	}

	return 1;
}

int OnFinnish(LX_PARSER *lx){
	LTPARSER *pClass;
	if(lx && lx->ExtenalData){
		pClass = (LTPARSER*)lx->ExtenalData;
		return pClass->OnFinnish(pClass);
	}
	return 1;
}

int OnTag(LX_PARSER *lx, unsigned char *tagname, unsigned char *content){
	LTPARSER *pClass;
	if(lx && lx->ExtenalData){
		pClass = (LTPARSER*)lx->ExtenalData;
		return pClass->OnTag(pClass, id, tagname, content);
	}
	return 1;
}

int OnTagEnd(LX_PARSER *lx, unsigned char *tagname){
	LTPARSER *pClass;
	if(lx && lx->ExtenalData){
		pClass = (LTPARSER*)lx->ExtenalData;
		return pClass->OnTagEnd(pClass, id, tagname);
	}
	return 1;
}

int OnAttribute(LX_PARSER *lx, unsigned char *tagname, unsigned char *attname, unsigned char *attvalue){
	LTPARSER *pClass;
	if(lx && lx->ExtenalData){
		pClass = (LTPARSER*)lx->ExtenalData;
		return pClass->OnAttribute(pClass, id, tagname, attname, attvalue);
	}
	return 1;
}

void OnError(LX_PARSER *lx, size_t line, char *error){
	LTPARSER *pClass;
	if(lx && lx->ExtenalData){
		pClass = (LTPARSER*)lx->ExtenalData;
		pClass->OnError(pClass, line, error);
	}
}

#ifdef __cplusplus
	 } /*extern "C" */
#endif

#endif /* LT_WRAPPER_HPP_ */
